namespace com.progsbase.no.inductive.libraries.foedselsnummer_v0_2_1{

using static System.Math;

public class BooleanArrayReference{
	public bool [] booleanArray;
}
public class BooleanReference{
	public bool booleanValue;
}
public class CharacterReference{
	public char characterValue;
}
public class NumberArrayReference{
	public double [] numberArray;
}
public class NumberReference{
	public double numberValue;
}
public class StringArrayReference{
	public StringReference [] stringArray;
}
public class StringReference{
	public char [] stringx;
}
public class datetimeDate{
	public double year;
	public double month;
	public double day;
}
public class datetimeDateReference{
	public datetimeDate date;
}
public class datetimeDateTimeTimezone{
	public datetimeDateTime dateTime;
	public double timezoneOffsetSeconds;
}
public class datetimeDateTimeTimezoneReference{
	public datetimeDateTimeTimezone dateTimeTimezone;
}
public class datetimeDateTime{
	public datetimeDate date;
	public double hours;
	public double minutes;
	public double seconds;
}
public class datetimeDateTimeReference{
	public datetimeDateTime dateTime;
}
public class foedselsnummer{
	public static bool IsValidNorwegianPersonalIdentificationNumber(char [] fnummer){
		bool gyldig;
		double d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11;
		datetimeDate aDatetimeDate;
		double k1, k2;
		NumberReference failures;
		StringReference errorMessage;

		errorMessage = CreateStringReference("".ToCharArray());

		if(fnummer.Length == 11d){
			failures = CreateNumberReference(0d);

			d1 = CharToNumberWithCheck(fnummer[0], failures);
			d2 = CharToNumberWithCheck(fnummer[1], failures);
			d3 = CharToNumberWithCheck(fnummer[2], failures);
			d4 = CharToNumberWithCheck(fnummer[3], failures);
			d5 = CharToNumberWithCheck(fnummer[4], failures);
			d6 = CharToNumberWithCheck(fnummer[5], failures);
			d7 = CharToNumberWithCheck(fnummer[6], failures);
			d8 = CharToNumberWithCheck(fnummer[7], failures);
			d9 = CharToNumberWithCheck(fnummer[8], failures);
			d10 = CharToNumberWithCheck(fnummer[9], failures);
			d11 = CharToNumberWithCheck(fnummer[10], failures);

			if(failures.numberValue == 0d){
				aDatetimeDate = GetDateFromNorwegianPersonalIdentificationNumber(fnummer, failures);

				if(datetimeIsValidDate(aDatetimeDate, errorMessage)){
					if(failures.numberValue == 0d){
						k1 = d1*3d + d2*7d + d3*6d + d4*1d + d5*8d + d6*9d + d7*4d + d8*5d + d9*2d;
						k1 = k1%11d;
						if(k1 != 0d){
							k1 = 11d - k1;
						}
						if(k1 == 10d){
							failures.numberValue = failures.numberValue + 1d;
						}

						k2 = d1*5d + d2*4d + d3*3d + d4*2d + d5*7d + d6*6d + d7*5d + d8*4d + d9*3d + k1*2d;
						k2 = k2%11d;
						if(k2 != 0d){
							k2 = 11d - k2;
						}
						if(k2 == 10d){
							failures.numberValue = failures.numberValue + 1d;
						}

						if(k1 == d10 && k2 == d11 && failures.numberValue == 0d){
							gyldig = true;
						}else{
							gyldig = false;
						}
					}else{
						gyldig = false;
					}
				}else{
					gyldig = false;
				}
			}else{
				gyldig = false;
			}
		}else{
			gyldig = false;
		}

		return gyldig;
	}


	public static datetimeDate GetDateFromNorwegianPersonalIdentificationNumber(char [] fnummer, NumberReference failures){
		datetimeDate aDatetimeDate;
		double individnummer;
		double day, month, year;
		double d1, d2, d3, d4, d5, d6, d7, d8, d9;

		aDatetimeDate = new datetimeDate();

		d1 = CharToNumberWithCheck(fnummer[0], failures);
		d2 = CharToNumberWithCheck(fnummer[1], failures);
		d3 = CharToNumberWithCheck(fnummer[2], failures);
		d4 = CharToNumberWithCheck(fnummer[3], failures);
		d5 = CharToNumberWithCheck(fnummer[4], failures);
		d6 = CharToNumberWithCheck(fnummer[5], failures);
		d7 = CharToNumberWithCheck(fnummer[6], failures);
		d8 = CharToNumberWithCheck(fnummer[7], failures);
		d9 = CharToNumberWithCheck(fnummer[8], failures);

		if(failures.numberValue == 0d){
			/* Individnummer*/
			individnummer = d7*100d + d8*10d + d9;

			/* Make date*/
			day = d1*10d + d2;
			month = d3*10d + d4;
			year = d5*10d + d6;

			if(individnummer >= 0d && individnummer <= 499d){
				year = year + 1900d;
			}else if(individnummer >= 500d && individnummer <= 749d && year >= 54d && year <= 99d){
				year = year + 1800d;
			}else if(individnummer >= 900d && individnummer <= 999d && year >= 40d && year <= 99d){
				year = year + 1900d;
			}else if(individnummer >= 500d && individnummer <= 999d && year >= 0d && year <= 39d){
				year = year + 1900d;
			}else{
				failures.numberValue = failures.numberValue + 1d;
			}

			aDatetimeDate.year = year;
			aDatetimeDate.month = month;
			aDatetimeDate.day = day;
		}

		return aDatetimeDate;
	}


	public static double CharToNumberWithCheck(char c, NumberReference failures){
		double val;

		if(charIsNumber(c)){
			val = nGetNumberFromNumberCharacterForBase(c, 10d);
		}else{
			val = 0d;
			failures.numberValue = failures.numberValue + 1d;
		}

		return val;
	}


	public static BooleanReference CreateBooleanReference(bool value){
		BooleanReference refx;
		refx = new BooleanReference();
		refx.booleanValue = value;

		return refx;
	}


	public static BooleanArrayReference CreateBooleanArrayReference(bool [] value){
		BooleanArrayReference refx;
		refx = new BooleanArrayReference();
		refx.booleanArray = value;

		return refx;
	}


	public static BooleanArrayReference CreateBooleanArrayReferenceLengthValue(double length, bool value){
		BooleanArrayReference refx;
		double i;
		refx = new BooleanArrayReference();
		refx.booleanArray = new bool [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.booleanArray[(int)(i)] = value;
		}

		return refx;
	}


	public static void FreeBooleanArrayReference(BooleanArrayReference booleanArrayReference){
		delete(booleanArrayReference.booleanArray);
		delete(booleanArrayReference);
	}


	public static CharacterReference CreateCharacterReference(char value){
		CharacterReference refx;
		refx = new CharacterReference();
		refx.characterValue = value;

		return refx;
	}


	public static NumberReference CreateNumberReference(double value){
		NumberReference refx;
		refx = new NumberReference();
		refx.numberValue = value;

		return refx;
	}


	public static NumberArrayReference CreateNumberArrayReference(double [] value){
		NumberArrayReference refx;
		refx = new NumberArrayReference();
		refx.numberArray = value;

		return refx;
	}


	public static NumberArrayReference CreateNumberArrayReferenceLengthValue(double length, double value){
		NumberArrayReference refx;
		double i;
		refx = new NumberArrayReference();
		refx.numberArray = new double [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.numberArray[(int)(i)] = value;
		}

		return refx;
	}


	public static void FreeNumberArrayReference(NumberArrayReference numberArrayReference){
		delete(numberArrayReference.numberArray);
		delete(numberArrayReference);
	}


	public static StringReference CreateStringReference(char [] value){
		StringReference refx;
		refx = new StringReference();
		refx.stringx = value;

		return refx;
	}


	public static StringReference CreateStringReferenceLengthValue(double length, char value){
		StringReference refx;
		double i;
		refx = new StringReference();
		refx.stringx = new char [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.stringx[(int)(i)] = value;
		}

		return refx;
	}


	public static void FreeStringReference(StringReference stringReference){
		delete(stringReference.stringx);
		delete(stringReference);
	}


	public static StringArrayReference CreateStringArrayReference(StringReference [] strings){
		StringArrayReference refx;
		refx = new StringArrayReference();
		refx.stringArray = strings;

		return refx;
	}


	public static StringArrayReference CreateStringArrayReferenceLengthValue(double length, char [] value){
		StringArrayReference refx;
		double i;
		refx = new StringArrayReference();
		refx.stringArray = new StringReference [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.stringArray[(int)(i)] = CreateStringReference(value);
		}

		return refx;
	}


	public static void FreeStringArrayReference(StringArrayReference stringArrayReference){
		double i;
		for(i = 0d; i < stringArrayReference.stringArray.Length; i = i + 1d){
			delete(stringArrayReference.stringArray[(int)(i)]);
		}
		delete(stringArrayReference.stringArray);
		delete(stringArrayReference);
	}


	public static char [] nCreateStringDecimalFromNumber(double decimalx){
		StringReference stringReference;
		stringReference = new StringReference();

		/* This will succeed because base = 10.*/
		nCreateStringFromNumberWithCheck(decimalx, 10d, stringReference);

		return stringReference.stringx;
	}


	public static bool nCreateStringFromNumberWithCheck(double decimalx, double basex, StringReference stringReference){
		char [] stringx;
		double maximumDigits;
		double digitPosition;
		bool hasPrintedPoint;
		double i, d;
		bool success;
		CharacterReference characterReference;
		char c;
		if(decimalx == 0d){
			stringReference.stringx = "0".ToCharArray();
			success = true;
		}else{
			characterReference = new CharacterReference();
			if(IsInteger(basex)){
				success = true;
				stringx = new char [0];
				maximumDigits = nGetMaximumDigitsForBase(basex);
				digitPosition = nGetFirstDigitPosition(decimalx, basex);
				decimalx = (double)Round(decimalx*Pow(basex, maximumDigits - digitPosition - 1d));
				hasPrintedPoint = false;
				if(digitPosition < 0d){
					stringx = AppendCharacter(stringx, '0');
					stringx = AppendCharacter(stringx, '.');
					hasPrintedPoint = true;
					for(i = 0d; i < -digitPosition - 1d; i = i + 1d){
						stringx = AppendCharacter(stringx, '0');
					}
				}
				for(i = 0d; i < maximumDigits && success; i = i + 1d){
					d = Floor(decimalx/Pow(basex, maximumDigits - i - 1d));
					if(!hasPrintedPoint && digitPosition - i + 1d == 0d){
						if(decimalx != 0d){
							stringx = AppendCharacter(stringx, '.');
						}
						hasPrintedPoint = true;
					}
					if(decimalx == 0d && hasPrintedPoint){
					}else{
						success = nGetSingleDigitCharacterFromNumberWithCheck(d, basex, characterReference);
						if(success){
							c = characterReference.characterValue;
							stringx = AppendCharacter(stringx, c);
						}
					}
					if(success){
						decimalx = decimalx - d*Pow(basex, maximumDigits - i - 1d);
					}
				}
				if(success){
					for(i = 0d; i < digitPosition - maximumDigits + 1d; i = i + 1d){
						stringx = AppendCharacter(stringx, '0');
					}
					stringReference.stringx = stringx;
				}
			}else{
				success = false;
			}
		}

		/* Done*/
		return success;
	}


	public static double nGetMaximumDigitsForBase(double basex){
		double t;
		t = Pow(10d, 15d);
		return Floor(Log10(t)/Log10(basex));
	}


	public static double nGetFirstDigitPosition(double decimalx, double basex){
		double power;
		double t;
		power = Ceiling(Log10(decimalx)/Log10(basex));

		t = decimalx*Pow(basex, -power);
		if(t < basex && t >= 1d){
		}else if(t >= basex){
			power = power + 1d;
		}else if(t < 1d){
			power = power - 1d;
		}

		return power;
	}


	public static bool nGetSingleDigitCharacterFromNumberWithCheck(double c, double basex, CharacterReference characterReference){
		char [] numberTable;
		bool success;
		numberTable = nGetDigitCharacterTable();

		if(c < basex || c < numberTable.Length){
			success = true;
			characterReference.characterValue = numberTable[(int)(c)];
		}else{
			success = false;
		}

		return success;
	}


	public static char [] nGetDigitCharacterTable(){
		char [] numberTable;
		numberTable = new char [36];

		numberTable[0] = '0';
		numberTable[1] = '1';
		numberTable[2] = '2';
		numberTable[3] = '3';
		numberTable[4] = '4';
		numberTable[5] = '5';
		numberTable[6] = '6';
		numberTable[7] = '7';
		numberTable[8] = '8';
		numberTable[9] = '9';
		numberTable[10] = 'A';
		numberTable[11] = 'B';
		numberTable[12] = 'C';
		numberTable[13] = 'D';
		numberTable[14] = 'E';
		numberTable[15] = 'F';
		numberTable[16] = 'G';
		numberTable[17] = 'H';
		numberTable[18] = 'I';
		numberTable[19] = 'J';
		numberTable[20] = 'K';
		numberTable[21] = 'L';
		numberTable[22] = 'M';
		numberTable[23] = 'N';
		numberTable[24] = 'O';
		numberTable[25] = 'P';
		numberTable[26] = 'Q';
		numberTable[27] = 'R';
		numberTable[28] = 'S';
		numberTable[29] = 'T';
		numberTable[30] = 'U';
		numberTable[31] = 'V';
		numberTable[32] = 'W';
		numberTable[33] = 'X';
		numberTable[34] = 'Y';
		numberTable[35] = 'Z';

		return numberTable;
	}


	public static bool nCreateNumberFromDecimalStringWithCheck(char [] stringx, NumberReference decimalReference, StringReference errorMessage){
		return nCreateNumberFromStringWithCheck(stringx, 10d, decimalReference, errorMessage);
	}


	public static double nCreateNumberFromDecimalString(char [] stringx){
		NumberReference doubleReference;
		StringReference stringReference;
		double number;
		doubleReference = CreateNumberReference(0d);
		stringReference = CreateStringReference("".ToCharArray());
		nCreateNumberFromStringWithCheck(stringx, 10d, doubleReference, stringReference);
		number = doubleReference.numberValue;

		delete(doubleReference);
		delete(stringReference);

		return number;
	}


	public static bool nCreateNumberFromStringWithCheck(char [] stringx, double basex, NumberReference decimalReference, StringReference errorMessage){
		char c;
		double i, j;
		bool isPositive;
		double beforeDecimalPoint, afterDecimalPoint;
		double n, validCharacters, d;
		bool success;
		success = true;
		i = 0d;
		isPositive = true;
		beforeDecimalPoint = 0d;
		afterDecimalPoint = 0d;
		n = 0d;
		validCharacters = 0d;

		if(basex >= 2d && basex <= 36d){
			for(j = 0d; j < stringx.Length; j = j + 1d){
				c = stringx[(int)(j)];
				if(nCharacterIsNumberCharacterInBase(c, basex) || c == '.' || c == '-'){
					validCharacters = validCharacters + 1d;
				}
			}
			if(validCharacters == stringx.Length){
				if(stringx.Length > 0d){
					c = stringx[(int)(i)];
					if(c == '-'){
						isPositive = false;
						i = i + 1d;
					}
					if(i < stringx.Length){
						c = stringx[(int)(i)];
						if(nCharacterIsNumberCharacterInBase(c, basex)){
							for(; nCharacterIsNumberCharacterInBase(c, basex) && (i < stringx.Length); ){
								beforeDecimalPoint = beforeDecimalPoint + 1d;
								i = i + 1d;
								if(i < stringx.Length){
									c = stringx[(int)(i)];
								}
							}
							if(i < stringx.Length){
								c = stringx[(int)(i)];
								if(c == '.'){
									i = i + 1d;
									if(i < stringx.Length){
										c = stringx[(int)(i)];
										for(; nCharacterIsNumberCharacterInBase(c, basex) && (i < stringx.Length); ){
											afterDecimalPoint = afterDecimalPoint + 1d;
											i = i + 1d;
											if(i < stringx.Length){
												c = stringx[(int)(i)];
											}
										}
									}else{
										success = false;
										errorMessage.stringx = "Number must have digits after the decimal point.".ToCharArray();
									}
								}
							}
						}else{
							success = false;
							errorMessage.stringx = "Number must start with digits (for negative numbers, after the negative sign).".ToCharArray();
						}
					}
					if(success != false){
						if(isPositive){
							i = 0d;
						}else{
							i = 1d;
						}
						for(j = 0d; j < beforeDecimalPoint; j = j + 1d){
							c = stringx[(int)(i)];
							i = i + 1d;
							d = nGetNumberFromNumberCharacterForBase(c, basex);
							n = n + d*Pow(basex, beforeDecimalPoint - j - 1d);
						}
						if(afterDecimalPoint > 0d){
							i = i + 1d;
							for(j = 0d; j < afterDecimalPoint; j = j + 1d){
								c = stringx[(int)(i)];
								i = i + 1d;
								d = nGetNumberFromNumberCharacterForBase(c, basex);
								n = n + d*Pow(basex, 0d - j - 1d);
							}
						}
						if(!isPositive){
							n = -n;
						}
						decimalReference.numberValue = n;
						success = true;
					}
				}else{
					success = false;
					errorMessage.stringx = "String has no content.".ToCharArray();
				}
			}else{
				success = false;
				errorMessage.stringx = "String contains invalid character.".ToCharArray();
			}
		}else{
			success = false;
			errorMessage.stringx = "Base must be from 2 to 36.".ToCharArray();
		}

		return success;
	}


	public static double nGetNumberFromNumberCharacterForBase(char c, double basex){
		char [] numberTable;
		double i;
		double position;
		numberTable = nGetDigitCharacterTable();
		position = 0d;

		for(i = 0d; i < basex; i = i + 1d){
			if(numberTable[(int)(i)] == c){
				position = i;
			}
		}

		return position;
	}


	public static bool nCharacterIsNumberCharacterInBase(char c, double basex){
		char [] numberTable;
		double i;
		bool found;
		numberTable = nGetDigitCharacterTable();
		found = false;

		for(i = 0d; i < basex; i = i + 1d){
			if(numberTable[(int)(i)] == c){
				found = true;
			}
		}

		return found;
	}


	public static double [] nStringToNumberArray(char [] str){
		NumberArrayReference numberArrayReference;
		StringReference stringReference;
		double [] numbers;
		numberArrayReference = new NumberArrayReference();
		stringReference = new StringReference();

		nStringToNumberArrayWithCheck(str, numberArrayReference, stringReference);

		numbers = numberArrayReference.numberArray;

		delete(numberArrayReference);
		delete(stringReference);

		return numbers;
	}


	public static bool nStringToNumberArrayWithCheck(char [] str, NumberArrayReference numberArrayReference, StringReference errorMessage){
		StringReference [] numberStrings;
		double [] numbers;
		double i;
		char [] numberString, trimmedNumberString;
		bool success;
		NumberReference numberReference;
		numberStrings = SplitByString(str, ",".ToCharArray());

		numbers = new double [(int)(numberStrings.Length)];
		success = true;
		numberReference = new NumberReference();

		for(i = 0d; i < numberStrings.Length; i = i + 1d){
			numberString = numberStrings[(int)(i)].stringx;
			trimmedNumberString = Trim(numberString);
			success = nCreateNumberFromDecimalStringWithCheck(trimmedNumberString, numberReference, errorMessage);
			numbers[(int)(i)] = numberReference.numberValue;
			FreeStringReference(numberStrings[(int)(i)]);
			delete(trimmedNumberString);
		}

		delete(numberStrings);
		delete(numberReference);

		numberArrayReference.numberArray = numbers;

		return success;
	}


	public static datetimeDate datetimeCreateDate(double year, double month, double day){
		datetimeDate date;
		date = new datetimeDate();

		date.year = year;
		date.month = month;
		date.day = day;

		return date;
	}


	public static bool datetimeIsLeapYearWithCheck(double year, BooleanReference isLeapYearReference, StringReference errorMessage){
		bool itIsLeapYear;
		bool success;
		if(year >= 1752d){
			success = true;
			itIsLeapYear = datetimeIsLeapYear(year);
		}else{
			success = false;
			itIsLeapYear = false;
			errorMessage.stringx = "Gregorian calendar was not in general use.".ToCharArray();
		}

		isLeapYearReference.booleanValue = itIsLeapYear;
		return success;
	}


	public static bool datetimeIsLeapYear(double year){
		bool itIsLeapYear;
		if(DivisibleBy(year, 4d)){
			if(DivisibleBy(year, 100d)){
				if(DivisibleBy(year, 400d)){
					itIsLeapYear = true;
				}else{
					itIsLeapYear = false;
				}
			}else{
				itIsLeapYear = true;
			}
		}else{
			itIsLeapYear = false;
		}

		return itIsLeapYear;
	}


	public static bool datetimeDayToDateWithCheck(double dayNr, datetimeDateReference dateReference, StringReference errorMessage){
		datetimeDate date;
		NumberReference remainder;
		bool success;
		if(dayNr >= -79623d){
			date = new datetimeDate();
			remainder = new NumberReference();
			remainder.numberValue = dayNr + 79623d;
			date.year = datetimeGetYearFromDayNr(remainder.numberValue, remainder);
			date.month = datetimeGetMonthFromDayNr(remainder.numberValue, date.year, remainder);
			date.day = 1d + remainder.numberValue;
			dateReference.date = date;
			success = true;
		}else{
			success = false;
			errorMessage.stringx = "Gregorian calendar was not in general use before 1752.".ToCharArray();
		}

		return success;
	}


	public static bool datetimeGetMonthFromDayNrWithCheck(double dayNr, double year, NumberReference monthReference, NumberReference remainderReference, StringReference errorMessage){
		double month;
		bool success;
		if(dayNr >= -79623d){
			month = datetimeGetMonthFromDayNr(dayNr, year, remainderReference);
			monthReference.numberValue = month;
			success = true;
		}else{
			success = false;
			errorMessage.stringx = "Gregorian calendar not in general use before 1752.".ToCharArray();
		}

		return success;
	}


	public static double datetimeGetMonthFromDayNr(double dayNr, double year, NumberReference remainderReference){
		double [] daysInMonth;
		bool done;
		double month;
		daysInMonth = datetimeGetDaysInMonth(year);
		done = false;
		month = 1d;

		for(; !done; ){
			if(dayNr > daysInMonth[(int)(month)]){
				dayNr = dayNr - daysInMonth[(int)(month)];
				month = month + 1d;
			}else{
				done = true;
			}
		}
		remainderReference.numberValue = dayNr;

		return month;
	}


	public static bool datetimeGetYearFromDayNrWithCheck(double dayNr, NumberReference yearReference, NumberReference remainder, StringReference errorMessage){
		bool success;
		double year;
		if(dayNr >= 0d){
			success = true;
			year = datetimeGetYearFromDayNr(dayNr, remainder);
			yearReference.numberValue = year;
		}else{
			success = false;
			errorMessage.stringx = "Day number must be 0 or higher. 0 is 1752-01-01.".ToCharArray();
		}

		return success;
	}


	public static double datetimeGetYearFromDayNr(double dayNr, NumberReference remainder){
		double nrOfDays;
		bool done;
		double year;
		done = false;
		year = 1752d;

		for(; !done; ){
			if(datetimeIsLeapYear(year)){
				nrOfDays = 366d;
			}else{
				nrOfDays = 365d;
			}
			if(dayNr >= nrOfDays){
				dayNr = dayNr - nrOfDays;
				year = year + 1d;
			}else{
				done = true;
			}
		}
		remainder.numberValue = dayNr;

		return year;
	}


	public static double datetimeDaysBetweenDates(datetimeDate A, datetimeDate B){
		double daysA, daysB, daysBetween;
		daysA = datetimeDateToDays(A);
		daysB = datetimeDateToDays(B);

		daysBetween = daysB - daysA;

		return daysBetween;
	}


	public static bool datetimeGetDaysInMonthWithCheck(double year, NumberArrayReference daysInMonthReference, StringReference errorMessage){
		double [] daysInMonth;
		bool success;
		datetimeDate date;
		date = datetimeCreateDate(year, 1d, 1d);

		success = datetimeIsValidDate(date, errorMessage);
		if(success){
			daysInMonth = datetimeGetDaysInMonth(year);
			daysInMonthReference.numberArray = daysInMonth;
		}

		return success;
	}


	public static double [] datetimeGetDaysInMonth(double year){
		double [] daysInMonth;
		daysInMonth = new double [(int)(1d + 12d)];

		daysInMonth[0] = 0d;
		daysInMonth[1] = 31d;

		if(datetimeIsLeapYear(year)){
			daysInMonth[2] = 29d;
		}else{
			daysInMonth[2] = 28d;
		}
		daysInMonth[3] = 31d;
		daysInMonth[4] = 30d;
		daysInMonth[5] = 31d;
		daysInMonth[6] = 30d;
		daysInMonth[7] = 31d;
		daysInMonth[8] = 31d;
		daysInMonth[9] = 30d;
		daysInMonth[10] = 31d;
		daysInMonth[11] = 30d;
		daysInMonth[12] = 31d;

		return daysInMonth;
	}


	public static bool datetimeDateToDaysWithCheck(datetimeDate date, NumberReference dayNumberReferenceReference, StringReference errorMessage){
		double days;
		bool success;
		success = datetimeIsValidDate(date, errorMessage);
		if(success){
			days = datetimeDateToDays(date);
			dayNumberReferenceReference.numberValue = days;
		}

		return success;
	}


	public static double datetimeDateToDays(datetimeDate date){
		double days;
		days = -79623d;

		days = days + datetimeDaysInYears(date.year);
		days = days + datetimeDaysInMonths(date.month, date.year);
		days = days + date.day - 1d;

		return days;
	}


	public static bool datetimeDateToWeekdayNumberWithCheck(datetimeDate date, NumberReference weekDayNumberReference, StringReference errorMessage){
		double weekDay;
		bool success;
		success = datetimeIsValidDate(date, errorMessage);
		if(success){
			weekDay = datetimeDateToWeekdayNumber(date);
			weekDayNumberReference.numberValue = weekDay;
		}

		return success;
	}


	public static double datetimeDateToWeekdayNumber(datetimeDate date){
		double days, weekDay;
		days = datetimeDateToDays(date);

		days = days + 79623d;
		days = days + 5d;

		weekDay = days%7d + 1d;

		return weekDay;
	}


	public static bool datetimeDaysInMonthsWithCheck(double month, double year, NumberReference daysInMonthsReference, StringReference errorMessage){
		double days;
		bool success;
		datetimeDate date;
		date = datetimeCreateDate(year, month, 1d);

		success = datetimeIsValidDate(date, errorMessage);
		if(success){
			days = datetimeDaysInMonths(month, year);
			daysInMonthsReference.numberValue = days;
		}

		return success;
	}


	public static double datetimeDaysInMonths(double month, double year){
		double [] daysInMonth;
		double days;
		double i;
		daysInMonth = datetimeGetDaysInMonth(year);

		days = 0d;
		for(i = 1d; i < month; i = i + 1d){
			days = days + daysInMonth[(int)(i)];
		}

		return days;
	}


	public static bool datetimeDaysInYearsWithCheck(double years, NumberReference daysReference, StringReference errorMessage){
		double days;
		bool success;
		datetimeDate date;
		date = datetimeCreateDate(years, 1d, 1d);

		success = datetimeIsValidDate(date, errorMessage);
		if(success){
			days = datetimeDaysInYears(years);
			daysReference.numberValue = days;
		}

		return success;
	}


	public static double datetimeDaysInYears(double years){
		double days;
		double i;
		double nrOfDays;
		days = 0d;
		for(i = 1752d; i < years; i = i + 1d){
			if(datetimeIsLeapYear(i)){
				nrOfDays = 366d;
			}else{
				nrOfDays = 365d;
			}
			days = days + nrOfDays;
		}

		return days;
	}


	public static bool datetimeIsValidDate(datetimeDate date, StringReference errorMessage){
		bool valid;
		double [] daysInMonth;
		double daysInThisMonth;
		if(date.year >= 1752d){
			if(date.month >= 1d && date.month <= 12d){
				daysInMonth = datetimeGetDaysInMonth(date.year);
				daysInThisMonth = daysInMonth[(int)(date.month)];
				if(date.day >= 1d && date.day <= daysInThisMonth){
					valid = true;
				}else{
					valid = false;
					errorMessage.stringx = "The month does not have the given day number.".ToCharArray();
				}
			}else{
				valid = false;
				errorMessage.stringx = "Month must be between 1 and 12, inclusive.".ToCharArray();
			}
		}else{
			valid = false;
			errorMessage.stringx = "Gregorian calendar was not in general use before 1752.".ToCharArray();
		}

		return valid;
	}


	public static datetimeDateTimeTimezone datetimeCreateDateTimeTimezone(double year, double month, double day, double hours, double minutes, double seconds, double timezoneOffsetSeconds){
		datetimeDateTimeTimezone dateTimeTimezone;
		dateTimeTimezone = new datetimeDateTimeTimezone();

		dateTimeTimezone.dateTime = datetimeCreateDateTime(year, month, day, hours, minutes, seconds);
		dateTimeTimezone.timezoneOffsetSeconds = timezoneOffsetSeconds;

		return dateTimeTimezone;
	}


	public static datetimeDateTimeTimezone datetimeCreateDateTimeTimezoneInHoursAndMinutes(double year, double month, double day, double hours, double minutes, double seconds, double timezoneOffsetHours, double timezoneOffsetMinutes){
		datetimeDateTimeTimezone dateTimeTimezone;
		dateTimeTimezone = new datetimeDateTimeTimezone();

		dateTimeTimezone.dateTime = datetimeCreateDateTime(year, month, day, hours, minutes, seconds);
		dateTimeTimezone.timezoneOffsetSeconds = datetimeGetSecondsFromHours(timezoneOffsetHours) + datetimeGetSecondsFromMinutes(timezoneOffsetMinutes);

		return dateTimeTimezone;
	}


	public static bool datetimeGetDateFromDateTimeTimeZone(datetimeDateTimeTimezone dateTimeTimezone, datetimeDateTimeReference dateTimeReference, StringReference errorMessage){
		datetimeDateTime dateTime;
		dateTime = dateTimeTimezone.dateTime;

		return datetimeAddSecondsToDateTime(dateTime, -dateTimeTimezone.timezoneOffsetSeconds, dateTimeReference, errorMessage);
	}


	public static bool datetimeCreateDateTimeTimezoneFromDateTimeAndTimeZoneInSeconds(datetimeDateTime dateTime, double timezoneOffsetSeconds, datetimeDateTimeTimezoneReference dateTimeTimezoneReference, StringReference errorMessage){
		bool success;
		datetimeDateTimeReference adjustedDateTimeReference;
		datetimeDateTimeTimezone dateTimeTimezone;
		adjustedDateTimeReference = new datetimeDateTimeReference();
		dateTimeTimezone = new datetimeDateTimeTimezone();

		success = datetimeAddSecondsToDateTime(dateTime, timezoneOffsetSeconds, adjustedDateTimeReference, errorMessage);

		if(success){
			dateTimeTimezone.dateTime = adjustedDateTimeReference.dateTime;
			dateTimeTimezone.timezoneOffsetSeconds = timezoneOffsetSeconds;
			dateTimeTimezoneReference.dateTimeTimezone = dateTimeTimezone;
		}

		return success;
	}


	public static bool datetimeCreateDateTimeTimezoneFromDateTimeAndTimeZoneInHoursAndMinutes(datetimeDateTime dateTime, double timezoneOffsetHours, double timezoneOffsetMinutes, datetimeDateTimeTimezoneReference dateTimeTimezoneReference, StringReference errorMessage){
		return datetimeCreateDateTimeTimezoneFromDateTimeAndTimeZoneInSeconds(dateTime, datetimeGetSecondsFromHours(timezoneOffsetHours) + datetimeGetSecondsFromMinutes(timezoneOffsetMinutes), dateTimeTimezoneReference, errorMessage);
	}


	public static datetimeDateTime datetimeCreateDateTime(double year, double month, double day, double hours, double minutes, double seconds){
		datetimeDateTime dateTime;
		dateTime = new datetimeDateTime();

		dateTime.date = datetimeCreateDate(year, month, day);
		dateTime.hours = hours;
		dateTime.minutes = minutes;
		dateTime.seconds = seconds;

		return dateTime;
	}


	public static bool datetimeGetDateTimeFromSeconds(double seconds, datetimeDateTimeReference dateTimeReference, StringReference errorMessage){
		datetimeDateTime dateTime;
		double secondsInMinute, secondsInHour, secondsInDay, days, remainder;
		datetimeDate date;
		datetimeDateReference dateReference;
		bool success;
		secondsInMinute = 60d;
		secondsInHour = 60d*secondsInMinute;
		secondsInDay = 24d*secondsInHour;
		days = Floor(seconds/secondsInDay);
		remainder = seconds - days*secondsInDay;
		dateReference = new datetimeDateReference();

		success = datetimeDayToDateWithCheck(days, dateReference, errorMessage);
		if(success){
			date = dateReference.date;
			dateTime = new datetimeDateTime();
			dateTime.date = date;
			dateTime.hours = Floor(remainder/secondsInHour);
			remainder = remainder - dateTime.hours*secondsInHour;
			dateTime.minutes = Floor(remainder/secondsInMinute);
			remainder = remainder - dateTime.minutes*secondsInMinute;
			dateTime.seconds = remainder;
			dateTimeReference.dateTime = dateTime;
		}

		return success;
	}


	public static double datetimeGetSecondsFromDateTime(datetimeDateTime dateTime){
		double seconds;
		double days;
		double secondsInMinute;
		double secondsInHour;
		double secondsInDay;
		NumberReference dayNumberReferenceReference;
		StringReference errorMessage;
		bool success;
		seconds = 0d;
		dayNumberReferenceReference = new NumberReference();
		errorMessage = new StringReference();

		success = datetimeDateToDaysWithCheck(dateTime.date, dayNumberReferenceReference, errorMessage);
		if(success){
			days = dayNumberReferenceReference.numberValue;
			secondsInMinute = 60d;
			secondsInHour = 60d*secondsInMinute;
			secondsInDay = 24d*secondsInHour;
			seconds = seconds + secondsInDay*days;
			seconds = seconds + secondsInHour*dateTime.hours;
			seconds = seconds + secondsInMinute*dateTime.minutes;
			seconds = seconds + dateTime.seconds;
		}

		return seconds;
	}


	public static double datetimeGetSecondsFromMinutes(double minutes){
		return minutes*60d;
	}


	public static double datetimeGetSecondsFromHours(double hours){
		return datetimeGetSecondsFromMinutes(hours*60d);
	}


	public static double datetimeGetSecondsFromDays(double days){
		return datetimeGetSecondsFromHours(days*24d);
	}


	public static double datetimeGetSecondsFromWeeks(double weeks){
		return datetimeGetSecondsFromDays(weeks*7d);
	}


	public static double datetimeGetMinutesFromSeconds(double seconds){
		return seconds/60d;
	}


	public static double datetimeGetHoursFromSeconds(double seconds){
		return datetimeGetMinutesFromSeconds(seconds)/60d;
	}


	public static double datetimeGetDaysFromSeconds(double seconds){
		return datetimeGetHoursFromSeconds(seconds)/24d;
	}


	public static double datetimeGetWeeksFromSeconds(double seconds){
		return datetimeGetDaysFromSeconds(seconds)/7d;
	}


	public static datetimeDate datetimeGetDateFromDateTime(datetimeDateTime dateTime){
		return dateTime.date;
	}


	public static bool datetimeAddSecondsToDateTime(datetimeDateTime dateTime, double seconds, datetimeDateTimeReference dateTimeReference, StringReference errorMessage){
		double secondsInDateTime;
		secondsInDateTime = datetimeGetSecondsFromDateTime(dateTime);
		secondsInDateTime = secondsInDateTime + seconds;

		return datetimeGetDateTimeFromSeconds(secondsInDateTime, dateTimeReference, errorMessage);
	}


	public static bool datetimeAddMinutesToDateTime(datetimeDateTime dateTime, double minutes, datetimeDateTimeReference dateTimeReference, StringReference errorMessage){
		return datetimeAddSecondsToDateTime(dateTime, datetimeGetSecondsFromMinutes(minutes), dateTimeReference, errorMessage);
	}


	public static bool datetimeAddHoursToDateTime(datetimeDateTime dateTime, double minutes, datetimeDateTimeReference dateTimeReference, StringReference errorMessage){
		return datetimeAddSecondsToDateTime(dateTime, datetimeGetSecondsFromHours(minutes), dateTimeReference, errorMessage);
	}


	public static bool datetimeAddDaysToDateTime(datetimeDateTime dateTime, double days, datetimeDateTimeReference dateTimeReference, StringReference errorMessage){
		return datetimeAddSecondsToDateTime(dateTime, datetimeGetSecondsFromDays(days), dateTimeReference, errorMessage);
	}


	public static bool datetimeAddWeeksToDateTime(datetimeDateTime dateTime, double weeks, datetimeDateTimeReference dateTimeReference, StringReference errorMessage){
		return datetimeAddSecondsToDateTime(dateTime, datetimeGetSecondsFromWeeks(weeks), dateTimeReference, errorMessage);
	}


	public static void AssertFalse(bool b, NumberReference failures){
		if(b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertTrue(bool b, NumberReference failures){
		if(!b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertEquals(double a, double b, NumberReference failures){
		if(a != b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertBooleansEqual(bool a, bool b, NumberReference failures){
		if(a != b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertCharactersEqual(char a, char b, NumberReference failures){
		if(a != b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertStringEquals(char [] a, char [] b, NumberReference failures){
		if(!StringsEqual(a, b)){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertNumberArraysEqual(double [] a, double [] b, NumberReference failures){
		double i;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length; i = i + 1d){
				AssertEquals(a[(int)(i)], b[(int)(i)], failures);
			}
		}else{
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertBooleanArraysEqual(bool [] a, bool [] b, NumberReference failures){
		double i;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length; i = i + 1d){
				AssertBooleansEqual(a[(int)(i)], b[(int)(i)], failures);
			}
		}else{
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertStringArraysEqual(StringReference [] a, StringReference [] b, NumberReference failures){
		double i;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length; i = i + 1d){
				AssertStringEquals(a[(int)(i)].stringx, b[(int)(i)].stringx, failures);
			}
		}else{
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static double [] StringToNumberArray(char [] stringx){
		double i;
		double [] array;
		array = new double [(int)(stringx.Length)];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			array[(int)(i)] = stringx[(int)(i)];
		}
		return array;
	}


	public static char [] NumberArrayToString(double [] array){
		double i;
		char [] stringx;
		stringx = new char [(int)(array.Length)];

		for(i = 0d; i < array.Length; i = i + 1d){
			stringx[(int)(i)] = (char)(array[(int)(i)]);
		}
		return stringx;
	}


	public static bool NumberArraysEqual(double [] a, double [] b){
		bool equal;
		double i;
		equal = true;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static bool BooleanArraysEqual(bool [] a, bool [] b){
		bool equal;
		double i;
		equal = true;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static bool StringsEqual(char [] a, char [] b){
		bool equal;
		double i;
		equal = true;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static void FillNumberArray(double [] a, double value){
		double i;
		for(i = 0d; i < a.Length; i = i + 1d){
			a[(int)(i)] = value;
		}
	}


	public static void FillString(char [] a, char value){
		double i;
		for(i = 0d; i < a.Length; i = i + 1d){
			a[(int)(i)] = value;
		}
	}


	public static void FillBooleanArray(bool [] a, bool value){
		double i;
		for(i = 0d; i < a.Length; i = i + 1d){
			a[(int)(i)] = value;
		}
	}


	public static bool FillNumberArrayInterval(double [] a, double value, double from, double to){
		double i;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length){
			for(i = from; i < to; i = i + 1d){
				a[(int)(i)] = value;
			}
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool FillBooleanArrayInterval(bool [] a, bool value, double from, double to){
		double i;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length){
			for(i = Max(from, 0d); i < Min(to, a.Length); i = i + 1d){
				a[(int)(i)] = value;
			}
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool FillStringInterval(char [] a, char value, double from, double to){
		double i;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length){
			for(i = Max(from, 0d); i < Min(to, a.Length); i = i + 1d){
				a[(int)(i)] = value;
			}
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static double [] CopyNumberArray(double [] a){
		double i;
		double [] n;
		n = new double [(int)(a.Length)];

		for(i = 0d; i < a.Length; i = i + 1d){
			n[(int)(i)] = a[(int)(i)];
		}

		return n;
	}


	public static bool [] CopyBooleanArray(bool [] a){
		double i;
		bool [] n;
		n = new bool [(int)(a.Length)];

		for(i = 0d; i < a.Length; i = i + 1d){
			n[(int)(i)] = a[(int)(i)];
		}

		return n;
	}


	public static char [] CopyString(char [] a){
		double i;
		char [] n;
		n = new char [(int)(a.Length)];

		for(i = 0d; i < a.Length; i = i + 1d){
			n[(int)(i)] = a[(int)(i)];
		}

		return n;
	}


	public static bool CopyNumberArrayRange(double [] a, double from, double to, NumberArrayReference copyReference){
		double i, length;
		double [] n;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length && from <= to){
			length = to - from;
			n = new double [(int)(length)];
			for(i = 0d; i < length; i = i + 1d){
				n[(int)(i)] = a[(int)(from + i)];
			}
			copyReference.numberArray = n;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool CopyBooleanArrayRange(bool [] a, double from, double to, BooleanArrayReference copyReference){
		double i, length;
		bool [] n;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length && from <= to){
			length = to - from;
			n = new bool [(int)(length)];
			for(i = 0d; i < length; i = i + 1d){
				n[(int)(i)] = a[(int)(from + i)];
			}
			copyReference.booleanArray = n;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool CopyStringRange(char [] a, double from, double to, StringReference copyReference){
		double i, length;
		char [] n;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length && from <= to){
			length = to - from;
			n = new char [(int)(length)];
			for(i = 0d; i < length; i = i + 1d){
				n[(int)(i)] = a[(int)(from + i)];
			}
			copyReference.stringx = n;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool IsLastElement(double length, double index){
		return index + 1d == length;
	}


	public static double [] CreateNumberArray(double length, double value){
		double [] array;
		array = new double [(int)(length)];
		FillNumberArray(array, value);

		return array;
	}


	public static bool [] CreateBooleanArray(double length, bool value){
		bool [] array;
		array = new bool [(int)(length)];
		FillBooleanArray(array, value);

		return array;
	}


	public static char [] CreateString(double length, char value){
		char [] array;
		array = new char [(int)(length)];
		FillString(array, value);

		return array;
	}


	public static bool SubstringWithCheck(char [] stringx, double from, double to, StringReference stringReference){
		bool success;
		if(from < stringx.Length && to < stringx.Length && from <= to && from >= 0d && to >= 0d){
			stringReference.stringx = Substring(stringx, from, to);
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static char [] Substring(char [] stringx, double from, double to){
		char [] n;
		double i;
		n = new char [(int)(Max(to - from, 0d))];

		for(i = from; i < to; i = i + 1d){
			n[(int)(i - from)] = stringx[(int)(i)];
		}

		return n;
	}


	public static char [] AppendString(char [] stringx, char [] s){
		char [] newString;
		newString = ConcatenateString(stringx, s);

		delete(stringx);

		return newString;
	}


	public static char [] ConcatenateString(char [] stringx, char [] s){
		char [] newString;
		double i;
		newString = new char [(int)(stringx.Length + s.Length)];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			newString[(int)(i)] = stringx[(int)(i)];
		}

		for(i = 0d; i < s.Length; i = i + 1d){
			newString[(int)(stringx.Length + i)] = s[(int)(i)];
		}

		return newString;
	}


	public static char [] AppendCharacter(char [] stringx, char c){
		char [] newString;
		newString = ConcatenateCharacter(stringx, c);

		delete(stringx);

		return newString;
	}


	public static char [] ConcatenateCharacter(char [] stringx, char c){
		char [] newString;
		double i;
		newString = new char [(int)(stringx.Length + 1d)];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			newString[(int)(i)] = stringx[(int)(i)];
		}

		newString[(int)(stringx.Length)] = c;

		return newString;
	}


	public static StringReference [] SplitByCharacter(char [] toSplit, char splitBy){
		StringReference [] split;
		char [] stringToSplitBy;
		stringToSplitBy = new char [1];
		stringToSplitBy[0] = splitBy;

		split = SplitByString(toSplit, stringToSplitBy);

		delete(stringToSplitBy);

		return split;
	}


	public static bool IndexOfCharacter(char [] stringx, char character, NumberReference indexReference){
		double i;
		bool found;
		found = false;
		for(i = 0d; i < stringx.Length && !found; i = i + 1d){
			if(stringx[(int)(i)] == character){
				found = true;
				indexReference.numberValue = i;
			}
		}

		return found;
	}


	public static bool SubstringEqualsWithCheck(char [] stringx, double from, char [] substring, BooleanReference equalsReference){
		bool success;
		if(from < stringx.Length){
			success = true;
			equalsReference.booleanValue = SubstringEquals(stringx, from, substring);
		}else{
			success = false;
		}

		return success;
	}


	public static bool SubstringEquals(char [] stringx, double from, char [] substring){
		double i;
		bool equal;
		equal = true;
		for(i = 0d; i < substring.Length && equal; i = i + 1d){
			if(stringx[(int)(from + i)] != substring[(int)(i)]){
				equal = false;
			}
		}

		return equal;
	}


	public static bool IndexOfString(char [] stringx, char [] substring, NumberReference indexReference){
		double i;
		bool found;
		found = false;
		for(i = 0d; i < stringx.Length - substring.Length + 1d && !found; i = i + 1d){
			if(SubstringEquals(stringx, i, substring)){
				found = true;
				indexReference.numberValue = i;
			}
		}

		return found;
	}


	public static bool ContainsCharacter(char [] stringx, char character){
		return IndexOfCharacter(stringx, character, new NumberReference());
	}


	public static bool ContainsString(char [] stringx, char [] substring){
		return IndexOfString(stringx, substring, new NumberReference());
	}


	public static void ToUpperCase(char [] stringx){
		double i;
		for(i = 0d; i < stringx.Length; i = i + 1d){
			stringx[(int)(i)] = charToUpperCase(stringx[(int)(i)]);
		}
	}


	public static void ToLowerCase(char [] stringx){
		double i;
		for(i = 0d; i < stringx.Length; i = i + 1d){
			stringx[(int)(i)] = charToLowerCase(stringx[(int)(i)]);
		}
	}


	public static bool EqualsIgnoreCase(char [] a, char [] b){
		bool equal;
		double i;
		if(a.Length == b.Length){
			equal = true;
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(charToLowerCase(a[(int)(i)]) != charToLowerCase(b[(int)(i)])){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static char [] ReplacesString(char [] stringx, char [] toReplace, char [] replaceWith){
		char [] result;
		double i;
		BooleanReference equalsReference;
		bool success;
		equalsReference = new BooleanReference();
		result = new char [0];

		for(i = 0d; i < stringx.Length; ){
			success = SubstringEqualsWithCheck(stringx, i, toReplace, equalsReference);
			if(success){
				success = equalsReference.booleanValue;
			}
			if(success && toReplace.Length > 0d){
				result = ConcatenateString(result, replaceWith);
				i = i + toReplace.Length;
			}else{
				result = ConcatenateCharacter(result, stringx[(int)(i)]);
				i = i + 1d;
			}
		}

		return result;
	}


	public static char [] ReplaceCharacter(char [] stringx, char toReplace, char replaceWith){
		char [] result;
		double i;
		result = new char [0];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			if(stringx[(int)(i)] == toReplace){
				result = ConcatenateCharacter(result, replaceWith);
			}else{
				result = ConcatenateCharacter(result, stringx[(int)(i)]);
			}
		}

		return result;
	}


	public static char [] Trim(char [] stringx){
		char [] result;
		double i, lastWhitespaceLocationStart, lastWhitespaceLocationEnd;
		bool firstNonWhitespaceFound;
		lastWhitespaceLocationStart = -1d;
		firstNonWhitespaceFound = false;
		for(i = 0d; i < stringx.Length && !firstNonWhitespaceFound; i = i + 1d){
			if(charIsWhiteSpace(stringx[(int)(i)])){
				lastWhitespaceLocationStart = i;
			}else{
				firstNonWhitespaceFound = true;
			}
		}

		/* Find whitepaces at the end.*/
		lastWhitespaceLocationEnd = stringx.Length;
		firstNonWhitespaceFound = false;
		for(i = stringx.Length - 1d; i >= 0d && !firstNonWhitespaceFound; i = i - 1d){
			if(charIsWhiteSpace(stringx[(int)(i)])){
				lastWhitespaceLocationEnd = i;
			}else{
				firstNonWhitespaceFound = true;
			}
		}

		if(lastWhitespaceLocationStart < lastWhitespaceLocationEnd){
			result = Substring(stringx, lastWhitespaceLocationStart + 1d, lastWhitespaceLocationEnd);
		}else{
			result = new char [0];
		}

		return result;
	}


	public static bool StartsWith(char [] stringx, char [] start){
		bool startsWithString;
		startsWithString = false;
		if(stringx.Length >= start.Length){
			startsWithString = SubstringEquals(stringx, 0d, start);
		}

		return startsWithString;
	}


	public static bool EndsWith(char [] stringx, char [] end){
		bool endsWithString;
		endsWithString = false;
		if(stringx.Length >= end.Length){
			endsWithString = SubstringEquals(stringx, stringx.Length - end.Length, end);
		}

		return endsWithString;
	}


	public static StringReference [] SplitByString(char [] toSplit, char [] splitBy){
		StringReference [] split;
		char [] next;
		double i;
		char c;
		StringReference n;
		split = new StringReference [0];

		next = new char [0];
		for(i = 0d; i < toSplit.Length; ){
			c = toSplit[(int)(i)];
			if(SubstringEquals(toSplit, i, splitBy)){
				if(split.Length != 0d || i != 0d){
					n = new StringReference();
					n.stringx = next;
					split = AddString(split, n);
					next = new char [0];
					i = i + splitBy.Length;
				}
			}else{
				next = AppendCharacter(next, c);
				i = i + 1d;
			}
		}

		if(next.Length > 0d){
			n = new StringReference();
			n.stringx = next;
			split = AddString(split, n);
		}

		return split;
	}


	public static double Negate(double x){
		return -x;
	}


	public static double Positive(double x){
		return +x;
	}


	public static double Factorial(double x){
		double i, f;
		f = 1d;

		for(i = 2d; i <= x; i = i + 1d){
			f = f*i;
		}

		return f;
	}


	public static double Roundx(double x){
		return Floor(x + 0.5);
	}


	public static double BankersRound(double x){
		double r;
		if(Absolute(x - Truncate(x)) == 0.5){
			if(!DivisibleBy(Roundx(x), 2d)){
				r = Roundx(x) - 1d;
			}else{
				r = Roundx(x);
			}
		}else{
			r = Roundx(x);
		}

		return r;
	}


	public static double Ceil(double x){
		return Ceiling(x);
	}


	public static double Floorx(double x){
		return Floor(x);
	}


	public static double Truncate(double x){
		double t;
		if(x >= 0d){
			t = Floor(x);
		}else{
			t = Ceiling(x);
		}

		return t;
	}


	public static double Absolute(double x){
		return Abs(x);
	}


	public static double Logarithm(double x){
		return Log10(x);
	}


	public static double NaturalLogarithm(double x){
		return Log(x);
	}


	public static double Sinx(double x){
		return Sin(x);
	}


	public static double Cosx(double x){
		return Cos(x);
	}


	public static double Tanx(double x){
		return Tan(x);
	}


	public static double Asinx(double x){
		return Asin(x);
	}


	public static double Acosx(double x){
		return Acos(x);
	}


	public static double Atanx(double x){
		return Atan(x);
	}


	public static double Atan2(double y, double x){
		double a;
		a = 0d;

		if(x > 0d){
			a = Atanx(y/x);
		}else if(x < 0d && y >= 0d){
			a = Atanx(y/x) + PI;
		}else if(x < 0d && y < 0d){
			a = Atanx(y/x) - PI;
		}else if(x == 0d && y > 0d){
			a = PI/2d;
		}else if(x == 0d && y < 0d){
			a = -PI/2d;
		}

		return a;
	}


	public static double Squareroot(double x){
		return Sqrt(x);
	}


	public static double Expx(double x){
		return Exp(x);
	}


	public static bool DivisibleBy(double a, double b){
		return ((a%b) == 0d);
	}


	public static double Combinations(double n, double k){
		return Factorial(n)/(Factorial(n - k)*Factorial(k));
	}


	public static bool EpsilonCompareApproximateDigits(double a, double b, double digits){
		double ad, bd, d, epsilon;
		bool ret;
		if(a < 0d && b < 0d || a > 0d && b > 0d){
			if(a < 0d && b < 0d){
				a = -a;
				b = -b;
			}
			ad = Log10(a);
			bd = Log10(b);
			d = Max(ad, bd);
			epsilon = Pow(10d, d - digits);
			ret = Abs(a - b) > epsilon;
		}else{
			ret = false;
		}

		return ret;
	}


	public static bool EpsilonCompare(double a, double b, double epsilon){
		return Abs(a - b) < epsilon;
	}


	public static double GreatestCommonDivisor(double a, double b){
		double t;
		for(; b != 0d; ){
			t = b;
			b = a%b;
			a = t;
		}

		return a;
	}


	public static bool IsInteger(double a){
		return (a - Floor(a)) == 0d;
	}


	public static bool GreatestCommonDivisorWithCheck(double a, double b, NumberReference gcdReference){
		bool success;
		double gcd;
		if(IsInteger(a) && IsInteger(b)){
			gcd = GreatestCommonDivisor(a, b);
			gcdReference.numberValue = gcd;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static double LeastCommonMultiple(double a, double b){
		double lcm;
		if(a > 0d && b > 0d){
			lcm = Abs(a*b)/GreatestCommonDivisor(a, b);
		}else{
			lcm = 0d;
		}

		return lcm;
	}


	public static double Sign(double a){
		double s;
		if(a > 0d){
			s = 1d;
		}else if(a < 0d){
			s = -1d;
		}else{
			s = 0d;
		}

		return s;
	}


	public static double Maxx(double a, double b){
		return Max(a, b);
	}


	public static double Minx(double a, double b){
		return Min(a, b);
	}


	public static double Power(double a, double b){
		return Pow(a, b);
	}


	public static double [] AddNumber(double [] list, double a){
		double [] newlist;
		double i;
		newlist = new double [(int)(list.Length + 1d)];
		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddNumberRef(NumberArrayReference list, double i){
		list.numberArray = AddNumber(list.numberArray, i);
	}


	public static double [] RemoveNumber(double [] list, double n){
		double [] newlist;
		double i;
		newlist = new double [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}
		
		delete(list);
		
		return newlist;
	}


	public static double GetNumberRef(NumberArrayReference list, double i){
		return list.numberArray[(int)(i)];
	}


	public static void RemoveNumberRef(NumberArrayReference list, double i){
		list.numberArray = RemoveNumber(list.numberArray, i);
	}


	public static StringReference [] AddString(StringReference [] list, StringReference a){
		StringReference [] newlist;
		double i;
		newlist = new StringReference [(int)(list.Length + 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddStringRef(StringArrayReference list, StringReference i){
		list.stringArray = AddString(list.stringArray, i);
	}


	public static StringReference [] RemoveString(StringReference [] list, double n){
		StringReference [] newlist;
		double i;
		newlist = new StringReference [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}
		
		delete(list);
		
		return newlist;
	}


	public static StringReference GetStringRef(StringArrayReference list, double i){
		return list.stringArray[(int)(i)];
	}


	public static void RemoveStringRef(StringArrayReference list, double i){
		list.stringArray = RemoveString(list.stringArray, i);
	}


	public static bool [] AddBoolean(bool [] list, bool a){
		bool [] newlist;
		double i;
		newlist = new bool [(int)(list.Length + 1d)];
		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddBooleanRef(BooleanArrayReference list, bool i){
		list.booleanArray = AddBoolean(list.booleanArray, i);
	}


	public static bool [] RemoveBoolean(bool [] list, double n){
		bool [] newlist;
		double i;
		newlist = new bool [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}
		
		delete(list);
		
		return newlist;
	}


	public static bool GetBooleanRef(BooleanArrayReference list, double i){
		return list.booleanArray[(int)(i)];
	}


	public static void RemoveDecimalRef(BooleanArrayReference list, double i){
		list.booleanArray = RemoveBoolean(list.booleanArray, i);
	}


	public static char [] AddCharacter(char [] list, char a){
		char [] newlist;
		double i;
		newlist = new char [(int)(list.Length + 1d)];
		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddCharacterRef(StringReference list, char i){
		list.stringx = AddCharacter(list.stringx, i);
	}


	public static char [] RemoveCharacter(char [] list, double n){
		char [] newlist;
		double i;
		newlist = new char [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}

		delete(list);

		return newlist;
	}


	public static char GetCharacterRef(StringReference list, double i){
		return list.stringx[(int)(i)];
	}


	public static void RemoveCharacterRef(StringReference list, double i){
		list.stringx = RemoveCharacter(list.stringx, i);
	}


	public static char charToLowerCase(char character){
		char toReturn;
		toReturn = character;
		if(character == 'A'){
			toReturn = 'a';
		}else if(character == 'B'){
			toReturn = 'b';
		}else if(character == 'C'){
			toReturn = 'c';
		}else if(character == 'D'){
			toReturn = 'd';
		}else if(character == 'E'){
			toReturn = 'e';
		}else if(character == 'F'){
			toReturn = 'f';
		}else if(character == 'G'){
			toReturn = 'g';
		}else if(character == 'H'){
			toReturn = 'h';
		}else if(character == 'I'){
			toReturn = 'i';
		}else if(character == 'J'){
			toReturn = 'j';
		}else if(character == 'K'){
			toReturn = 'k';
		}else if(character == 'L'){
			toReturn = 'l';
		}else if(character == 'M'){
			toReturn = 'm';
		}else if(character == 'N'){
			toReturn = 'n';
		}else if(character == 'O'){
			toReturn = 'o';
		}else if(character == 'P'){
			toReturn = 'p';
		}else if(character == 'Q'){
			toReturn = 'q';
		}else if(character == 'R'){
			toReturn = 'r';
		}else if(character == 'S'){
			toReturn = 's';
		}else if(character == 'T'){
			toReturn = 't';
		}else if(character == 'U'){
			toReturn = 'u';
		}else if(character == 'V'){
			toReturn = 'v';
		}else if(character == 'W'){
			toReturn = 'w';
		}else if(character == 'X'){
			toReturn = 'x';
		}else if(character == 'Y'){
			toReturn = 'y';
		}else if(character == 'Z'){
			toReturn = 'z';
		}

		return toReturn;
	}


	public static char charToUpperCase(char character){
		char toReturn;
		toReturn = character;
		if(character == 'a'){
			toReturn = 'A';
		}else if(character == 'b'){
			toReturn = 'B';
		}else if(character == 'c'){
			toReturn = 'C';
		}else if(character == 'd'){
			toReturn = 'D';
		}else if(character == 'e'){
			toReturn = 'E';
		}else if(character == 'f'){
			toReturn = 'F';
		}else if(character == 'g'){
			toReturn = 'G';
		}else if(character == 'h'){
			toReturn = 'H';
		}else if(character == 'i'){
			toReturn = 'I';
		}else if(character == 'j'){
			toReturn = 'J';
		}else if(character == 'k'){
			toReturn = 'K';
		}else if(character == 'l'){
			toReturn = 'L';
		}else if(character == 'm'){
			toReturn = 'M';
		}else if(character == 'n'){
			toReturn = 'N';
		}else if(character == 'o'){
			toReturn = 'O';
		}else if(character == 'p'){
			toReturn = 'P';
		}else if(character == 'q'){
			toReturn = 'Q';
		}else if(character == 'r'){
			toReturn = 'R';
		}else if(character == 's'){
			toReturn = 'S';
		}else if(character == 't'){
			toReturn = 'T';
		}else if(character == 'u'){
			toReturn = 'U';
		}else if(character == 'v'){
			toReturn = 'V';
		}else if(character == 'w'){
			toReturn = 'W';
		}else if(character == 'x'){
			toReturn = 'X';
		}else if(character == 'y'){
			toReturn = 'Y';
		}else if(character == 'z'){
			toReturn = 'Z';
		}

		return toReturn;
	}


	public static bool charIsUpperCase(char character){
		bool isUpper;
		isUpper = false;
		if(character == 'A'){
			isUpper = true;
		}else if(character == 'B'){
			isUpper = true;
		}else if(character == 'C'){
			isUpper = true;
		}else if(character == 'D'){
			isUpper = true;
		}else if(character == 'E'){
			isUpper = true;
		}else if(character == 'F'){
			isUpper = true;
		}else if(character == 'G'){
			isUpper = true;
		}else if(character == 'H'){
			isUpper = true;
		}else if(character == 'I'){
			isUpper = true;
		}else if(character == 'J'){
			isUpper = true;
		}else if(character == 'K'){
			isUpper = true;
		}else if(character == 'L'){
			isUpper = true;
		}else if(character == 'M'){
			isUpper = true;
		}else if(character == 'N'){
			isUpper = true;
		}else if(character == 'O'){
			isUpper = true;
		}else if(character == 'P'){
			isUpper = true;
		}else if(character == 'Q'){
			isUpper = true;
		}else if(character == 'R'){
			isUpper = true;
		}else if(character == 'S'){
			isUpper = true;
		}else if(character == 'T'){
			isUpper = true;
		}else if(character == 'U'){
			isUpper = true;
		}else if(character == 'V'){
			isUpper = true;
		}else if(character == 'W'){
			isUpper = true;
		}else if(character == 'X'){
			isUpper = true;
		}else if(character == 'Y'){
			isUpper = true;
		}else if(character == 'Z'){
			isUpper = true;
		}

		return isUpper;
	}


	public static bool charIsLowerCase(char character){
		bool isLower;
		isLower = false;
		if(character == 'a'){
			isLower = true;
		}else if(character == 'b'){
			isLower = true;
		}else if(character == 'c'){
			isLower = true;
		}else if(character == 'd'){
			isLower = true;
		}else if(character == 'e'){
			isLower = true;
		}else if(character == 'f'){
			isLower = true;
		}else if(character == 'g'){
			isLower = true;
		}else if(character == 'h'){
			isLower = true;
		}else if(character == 'i'){
			isLower = true;
		}else if(character == 'j'){
			isLower = true;
		}else if(character == 'k'){
			isLower = true;
		}else if(character == 'l'){
			isLower = true;
		}else if(character == 'm'){
			isLower = true;
		}else if(character == 'n'){
			isLower = true;
		}else if(character == 'o'){
			isLower = true;
		}else if(character == 'p'){
			isLower = true;
		}else if(character == 'q'){
			isLower = true;
		}else if(character == 'r'){
			isLower = true;
		}else if(character == 's'){
			isLower = true;
		}else if(character == 't'){
			isLower = true;
		}else if(character == 'u'){
			isLower = true;
		}else if(character == 'v'){
			isLower = true;
		}else if(character == 'w'){
			isLower = true;
		}else if(character == 'x'){
			isLower = true;
		}else if(character == 'y'){
			isLower = true;
		}else if(character == 'z'){
			isLower = true;
		}

		return isLower;
	}


	public static bool charIsLetter(char character){
		return charIsUpperCase(character) || charIsLowerCase(character);
	}


	public static bool charIsNumber(char character){
		bool isNumber;
		isNumber = false;
		if(character == '0'){
			isNumber = true;
		}else if(character == '1'){
			isNumber = true;
		}else if(character == '2'){
			isNumber = true;
		}else if(character == '3'){
			isNumber = true;
		}else if(character == '4'){
			isNumber = true;
		}else if(character == '5'){
			isNumber = true;
		}else if(character == '6'){
			isNumber = true;
		}else if(character == '7'){
			isNumber = true;
		}else if(character == '8'){
			isNumber = true;
		}else if(character == '9'){
			isNumber = true;
		}

		return isNumber;
	}


	public static bool charIsWhiteSpace(char character){
		bool isWhiteSpace;
		isWhiteSpace = false;
		if(character == ' '){
			isWhiteSpace = true;
		}else if(character == '\t'){
			isWhiteSpace = true;
		}else if(character == '\n'){
			isWhiteSpace = true;
		}else if(character == '\r'){
			isWhiteSpace = true;
		}

		return isWhiteSpace;
	}


	public static bool charIsSymbol(char character){
		bool isSymbol;
		isSymbol = false;
		if(character == '!'){
			isSymbol = true;
		}else if(character == '\"'){
			isSymbol = true;
		}else if(character == '#'){
			isSymbol = true;
		}else if(character == '$'){
			isSymbol = true;
		}else if(character == '%'){
			isSymbol = true;
		}else if(character == '&'){
			isSymbol = true;
		}else if(character == '\''){
			isSymbol = true;
		}else if(character == '('){
			isSymbol = true;
		}else if(character == ')'){
			isSymbol = true;
		}else if(character == '*'){
			isSymbol = true;
		}else if(character == '+'){
			isSymbol = true;
		}else if(character == ','){
			isSymbol = true;
		}else if(character == '-'){
			isSymbol = true;
		}else if(character == '.'){
			isSymbol = true;
		}else if(character == '/'){
			isSymbol = true;
		}else if(character == ':'){
			isSymbol = true;
		}else if(character == ';'){
			isSymbol = true;
		}else if(character == '<'){
			isSymbol = true;
		}else if(character == '='){
			isSymbol = true;
		}else if(character == '>'){
			isSymbol = true;
		}else if(character == '?'){
			isSymbol = true;
		}else if(character == '@'){
			isSymbol = true;
		}else if(character == '['){
			isSymbol = true;
		}else if(character == '\\'){
			isSymbol = true;
		}else if(character == ']'){
			isSymbol = true;
		}else if(character == '^'){
			isSymbol = true;
		}else if(character == '_'){
			isSymbol = true;
		}else if(character == '`'){
			isSymbol = true;
		}else if(character == '{'){
			isSymbol = true;
		}else if(character == '|'){
			isSymbol = true;
		}else if(character == '}'){
			isSymbol = true;
		}else if(character == '~'){
			isSymbol = true;
		}

		return isSymbol;
	}


	public static void delete(System.Object objectx){
		// C# has garbage collection.
	}
}

}